local name, ns = ...

local class = select(2, UnitClass("player"))
if class ~= "MONK" and class ~= "WARLOCK" then return end

local PORT_BUFF_NAME = class == "MONK" and GetSpellInfo(101643) or GetSpellInfo(48018)
local DB
local char_key = UnitName("player").."-"..GetRealmName()
local Portal = {}
local distance_format
local pX, pY

local atan2 = math.atan2
local sqrt = sqrt
--local UnitHasVehicleUI = UnitHasVehicleUI
--local IsInPetBattle = C_PetBattles.IsInBattle
local abs = math.abs
local str_fmt = string.format
local GetPlayerFacing = GetPlayerFacing

local defaults = {
	MONK = {
		texpath_body = [[Interface\AddOns\Teleportie\Media\Textures\Transcendence1.blp]],
		texpath_direction = [[Interface\AddOns\Teleportie\Media\Textures\Direction1.blp]],
		texpath_bg = [[Interface\AddOns\Teleportie\Media\Textures\Background1.blp]],
		texColor_far = {255/255, 15/255, 0/255, .5}, --red
		texColor_near = {83/255, 255/255, 197/255, 1}, --green
		textColor_far = {255/255, 15/255, 0/255}, --red
		textColor_near = {20/255, 255/255, 80/255}, --green	
		decimals = 1,
		insets = 0,
		frame_size = 48,
		font_height = 12,
		Portal = {},
	},
	WARLOCK = {
	}
}
defaults.WARLOCK = defaults.MONK

local function UpdateOldVersionSettings()
	for k, v in pairs(defaults[class]) do
		if DB[k] == nil then
			DB[k] = v
		end
	end
end

local f = CreateFrame("FRAME", name, UIParent)
f:Hide()
f:SetSize(48, 48)
f:SetPoint("CENTER")
f.Tex_Icon = f:CreateTexture(nil, "BORDER")
f.Tex_Icon:SetAllPoints()
f.Tex_Circle = f:CreateTexture(nil, "BACKGROUND")
f.Tex_Circle:SetAllPoints()
f.FS_Distance = f:CreateFontString(nil, "BORDER")
f.FS_Distance:SetFont([[Fonts\FRIZQT__.TTF]], 12, "OUTLINE")
f.FS_Distance:SetNonSpaceWrap(true)
f.FS_Distance:SetJustifyH("CENTER")
f.FS_Distance:SetPoint("BOTTOM", f, "BOTTOM", 0, 12)
f.Direction = f:CreateTexture(nil, "ARTWORK")
f.Direction:SetPoint("CENTER")
f.Direction:SetAllPoints()
f.Dir_Ag = f.Direction:CreateAnimationGroup()
f.Dir_Rot = f.Dir_Ag:CreateAnimation("Rotation")
f.Dir_Rot:SetDuration(0)
f.Dir_Rot:SetEndDelay(10)
f.Dir_Ag:SetLooping("REPEAT")
f.Dir_Ag:Play()
f.Dir_Ag:Pause()
f:SetClampedToScreen(true)
f:EnableMouse(true)
f:SetMovable(true)
f:RegisterForDrag("RightButton")
f:SetFrameStrata("HIGH")
f:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "player")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("PLAYER_ALIVE")
f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ZONE_CHANGED_NEW_AREA")

local CheckPortalBuff

local function CheckPortalBuffFade()
	if not UnitBuff("player", PORT_BUFF_NAME) then
		f:Hide()
		wipe(Portal)
		f:UnregisterEvent("UNIT_AURA")
	end
end

local function CheckPortalBuffApply()
	if UnitBuff("player", PORT_BUFF_NAME) then
		CheckPortalBuff = CheckPortalBuffFade
	end
end

function f:UpdateAppearance(s)
	f.Tex_Icon:SetTexture(s.texpath_body)
	f.Tex_Circle:SetTexture(s.texpath_bg)
	f.Direction:SetTexture(s.texpath_direction)
	local insets = s.insets
	f:SetClampRectInsets(-1 * insets, insets, insets, -1 * insets)
	f.FS_Distance:SetTextHeight(s.font_height)
	distance_format = "%00000."..s.decimals.."f"
	f:SetSize(s.frame_size, s.frame_size)
	if s.framePos then
		self:SetPoint(unpack(s.framePos))
	else
		self:SetPoint("CENTER", UIParent, "CENTER")
	end
	f:SetSize(s.frame_size, s.frame_size)
end

local function MarkPosition(savedX, savedY, savedInstanceId)
--call with parameters to restore saved info or without to mark a new one
	local instance = select(8, GetInstanceInfo())
	if savedInstanceId and savedInstanceId ~= instance then wipe(Portal) return end

	Portal.instance = instance

	local x, y = UnitPosition("player")

	pX, pY = savedX or x, savedY or y
	Portal.RealX, Portal.RealY = pX, pY

	f:Show()
	CheckPortalBuff = CheckPortalBuffApply
	f:RegisterUnitEvent("UNIT_AURA", "player")
end

--User interface
local anchorOpposites = {
	TOPLEFT     = "ANCHOR_BOTTOMRIGHT",
	TOPRIGHT    = "ANCHOR_BOTTOMLEFT",
	BOTTOMLEFT  = "ANCHOR_TOPRIGHT",
	BOTTOMRIGHT = "ANCHOR_TOPLEFT",
	TOP         = "ANCHOR_BOTTOM",
	BOTTOM      = "ANCHOR_TOP",
	LEFT        = "ANCHOR_RIGHT",
	RIGHT       = "ANCHOR_LEFT",
	CENTER      = "ANCHOR_TOP",
}
f:SetScript("OnEnter", function(self)
	GameTooltip:SetOwner(self, anchorOpposites[DB.framePos[1]])
	GameTooltip:ClearLines()
	GameTooltip:AddLine("Teleportie")
	GameTooltip:AddLine("Move me by shift + right mouse draging!")
	GameTooltip:AddLine("Scale me by mouse wheel!")
	--GameTooltip:AddLine("For more options type: /teleportie")
	GameTooltip:Show()
end)
f:SetScript("OnLeave", function(self)
	--GameTooltip:FadeOut()
	GameTooltip:Hide()
end)
f:SetScript("OnDragStart", function(self, ...)
	if IsShiftKeyDown() then
		self:StartMoving()
	end
end)
f:SetScript("OnDragStop", function(self, ...)
	self:StopMovingOrSizing()
	DB.framePos = {self:GetPoint()}
end)
f:SetScript("OnEvent", function(self, event, ...) 
	if event == "UNIT_AURA" then
		CheckPortalBuff()
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		local spellId = select(5, ...)
		if spellId == 101643 or spellId == 48018 or spellId == 119996 then
			MarkPosition()
		end
	elseif event == "PLAYER_LOGIN" then
		Teleportie_DB = Teleportie_DB or {}
		Teleportie_DB[char_key] = Teleportie_DB[char_key] or defaults[class]
		DB = Teleportie_DB[char_key]
		UpdateOldVersionSettings()
		f:UpdateAppearance(DB)
		Portal = DB.Portal
	elseif (event == "PLAYER_ENTERING_WORLD" or event == "ZONE_CHANGED_NEW_AREA") and Portal.RealX then
		MarkPosition(Portal.RealX, Portal.RealY, Portal.instance)
	elseif event == "PLAYER_ALIVE" then
		self:Hide()
		wipe(Portal)
	end
end)
f:SetScript("OnMouseWheel", function(self, delta)
	local size = self:GetSize() + (0.5 * delta)
	self:SetSize(size, size)
	DB.frame_size = size
end)

local old_in_range
local function OnUpdate() 	
	local x, y = UnitPosition("player")
	local x_diff, y_diff = pX - x, pY - y
	local dist = sqrt(x_diff ^ 2 + y_diff ^ 2)
	local facing = GetPlayerFacing()
	local direction = atan2(y_diff, x_diff)
	f.FS_Distance:SetText(str_fmt(distance_format, dist))
	f.Dir_Rot:SetRadians(direction - facing)

	local in_range = dist < 39.99
	if in_range ~= old_in_range then
		if in_range then
			f.Tex_Icon:SetVertexColor(unpack(DB.texColor_near))
			f.FS_Distance:SetTextColor(unpack(DB.textColor_near))
		else
			f.Tex_Icon:SetVertexColor(unpack(DB.texColor_far))
			f.FS_Distance:SetTextColor(unpack(DB.textColor_far))
		end
	end
	old_in_range = in_range
end
f:SetScript("OnUpdate", OnUpdate)

SLASH_TELEPORTIE1 = "/teleportie"
SlashCmdList["TELEPORTIE"] = function(msg)
	
end